﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Net;
using System.Threading;
using System.Windows;
using System.Windows.Navigation;
using BeamNG_map_options.Screens;

namespace BeamNG_map_options
{
    public partial class MainWindow : Window
    {
        private Dictionary<string, object> dataDictionary = new Dictionary<string, object>(); //Data sharing across windows. Main window stores it, children retrieve it and save it.
        private string log = ""; //Log string for use during installation and viewing.
        private double version = 0.3; //This version, check against the web version and see if we need updates.

        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Initialized(object sender, EventArgs e)
        {
            pageFrame.Navigate(new WelcomeScreen(this)); //First thing we want to do is navigate to the welcome screen on startup.
            CheckForUpdate(); //Then we check for an update in another thread.

            this.Title = "BeamNG Map Options (version " + version + ")"; //Set the title to the version number we have.
        }

        /// <summary>
        /// Enable or disable the next button on the main window.
        /// </summary>
        /// <param name="enable">Whether you want to enable or disable the button.</param>
        public void EnableNextButton(bool enable)
        {
            buttonNext.IsEnabled = enable;
        }

        /// <summary>
        /// Enable or disable the back button on the main window.
        /// </summary>
        /// <param name="enable">Whether you want to enable or disable the button.</param>
        public void EnableBackButton(bool enable)
        {
            buttonBack.IsEnabled = enable;
        }

        /// <summary>
        /// Handles navigation between child windows. This window will change the content to
        /// the selection you make.
        /// </summary>
        /// <param name="content">The object you wish to navigate to.</param>
        public void Navigate(object content)
        {
            pageFrame.Navigate(content);
        }

        /// <summary>
        /// Navigate back to the previous child window.
        /// </summary>
        public void GoBack()
        {
            if (pageFrame.NavigationService.CanGoBack)
            {
                pageFrame.NavigationService.GoBack();
            }
        }

        /// <summary>
        /// Get the selected index of already stored data.
        /// This returns null if there is no data already in this index.
        /// </summary>
        /// <param name="index">The index of the data to get.</param>
        /// <returns></returns>
        public object GetData(string index)
        {
            if (dataDictionary.ContainsKey(index))
            {
                return dataDictionary[index];
            }

            return null;
        }

        /// <summary>
        /// Set the selected index to store data for use in another window later.
        /// </summary>
        /// <param name="index">The key to store.</param>
        /// <param name="data">The data to store associated with the key.</param>
        public void SetData(string index, object data)
        {
            dataDictionary[index] = data;
        }

        /// <summary>
        /// Returns the log string for use when outputting to the screen for the user.
        /// </summary>
        /// <returns>Current stored log.</returns>
        public string GetLog()
        {
            return log;
        }

        /// <summary>
        /// Adds a new line to the existing log.
        /// </summary>
        /// <param name="line">The line to add.</param>
        public void AddLineToLog(string line)
        {
            log += "\r\n" + line;
        }

        /// <summary>
        /// Completely clears the log to be written to again.
        /// </summary>
        public void ClearLog()
        {
            log = "";
        }

        /// <summary>
        /// If the next button is clicked.
        /// </summary>
        private void buttonNext_Click(object sender, RoutedEventArgs e)
        {
            string navName = pageFrame.NavigationService.Content.GetType().Name; //Get the current name of the child window we are on.
            object currentNav = pageFrame.NavigationService.Content; //Get the current navigation content.

            //Reset the buttons so each window can handle it themselves with the same outcome.
            EnableNextButton(true);
            EnableBackButton(true);

            //Now we go through all the names of child windows and match it up to the current one.
            switch( navName )
            {
                case "WelcomeScreen":
                    pageFrame.Navigate(new DirectoryScreen(this));
                    break;
                case "DirectoryScreen":
                    SetData("directory", ((DirectoryScreen)currentNav).GetDirectoryName());
                    pageFrame.Navigate(new OptionsScreen(this));
                    break;
                case "DupeLevelScreen":
                    SetData("newName", ((DupeLevelScreen)currentNav).mapName.Text);
                    SetData("fixMatLoc", ((DupeLevelScreen)currentNav).fixMat.IsChecked.Value);
                    SetData("copyMatLoc", ((DupeLevelScreen)currentNav).copyMat.IsChecked.Value);
                    SetData("fixShapeLoc", ((DupeLevelScreen)currentNav).fixShape.IsChecked.Value);
                    SetData("copyShapeLoc", ((DupeLevelScreen)currentNav).copyShape.IsChecked.Value);
                    SetData("fixForestLoc", ((DupeLevelScreen)currentNav).fixForest.IsChecked.Value);
                    SetData("copyForestLoc", ((DupeLevelScreen)currentNav).copyForest.IsChecked.Value);
                    SetData("fixUnknown", ((DupeLevelScreen)currentNav).fixUnknown.IsChecked.Value);
                    SetData("recoverTerrain", ((DupeLevelScreen)currentNav).recoverTerrain.IsChecked.Value);
                    SetData("fixNoExtension", ((DupeLevelScreen)currentNav).fixNoExtension.IsChecked.Value);
                    pageFrame.Navigate(new CopyDataScreen(this));
                    break;
                case "CopyOptionsScreen":
                    SetData("newName", ((RenameOptionsScreen)currentNav).mapName.Text);
                    SetData("fixMatLoc", ((RenameOptionsScreen)currentNav).fixMat.IsChecked.Value);
                    SetData("copyMatLoc", ((RenameOptionsScreen)currentNav).copyMat.IsChecked.Value);
                    SetData("fixShapeLoc", ((RenameOptionsScreen)currentNav).fixShape.IsChecked.Value);
                    SetData("copyShapeLoc", ((RenameOptionsScreen)currentNav).copyShape.IsChecked.Value);
                    SetData("fixForestLoc", ((RenameOptionsScreen)currentNav).fixForest.IsChecked.Value);
                    SetData("copyForestLoc", ((RenameOptionsScreen)currentNav).copyForest.IsChecked.Value);
                    SetData("fixUnknown", ((RenameOptionsScreen)currentNav).fixUnknown.IsChecked.Value);
                    SetData("recoverTerrain", ((RenameOptionsScreen)currentNav).recoverTerrain.IsChecked.Value);
                    SetData("fixNoExtension", ((RenameOptionsScreen)currentNav).fixNoExtension.IsChecked.Value);
                    pageFrame.Navigate(new RenameScreen(this));
                    break;
                case "RemoveArtOptionsScreen":
                    SetData("removeModels", ((RemoveArtOptionsScreen)currentNav).removeModels.IsChecked.Value);
                    SetData("removeTextures", ((RemoveArtOptionsScreen)currentNav).removeTextures.IsChecked.Value);
                    pageFrame.Navigate(new RemoveArtScreen(this));
                    break;
            }
        }

        /// <summary>
        /// Cancel button clicked, exit program.
        /// </summary>
        private void buttonCancel_Click(object sender, RoutedEventArgs e)
        {
            MessageBoxResult result = MessageBox.Show("Are you sure you want to close this wizard?", "Are you sure?", MessageBoxButton.YesNo, MessageBoxImage.Question);

            if (result == MessageBoxResult.Yes)
            {
                this.Close();
            }
        }

        /// <summary>
        /// Back button clicked. Goes back to the previous window if possible.
        /// </summary>
        private void buttonBack_Click(object sender, RoutedEventArgs e)
        {
            if (pageFrame.CanGoBack)
            {
                pageFrame.GoBack();
            }
        }

        /// <summary>
        /// When the page frame is updated.
        /// </summary>
        private void pageFrame_Navigated(object sender, NavigationEventArgs e)
        {
            if (pageFrame.CanGoBack) //Enable/disable the back button if you can or can't go back.
            {
                buttonBack.IsEnabled = true;
            }
            else
            {
                buttonBack.IsEnabled = false;
            }
        }

        /// <summary>
        /// Update thread, checks for an update online and reports back with the result.
        /// </summary>
        private void CheckForUpdate()
        {
            Thread versionThread = new Thread(new ThreadStart(delegate
            {
                WebClient client = new WebClient(); //Initiate a new web client.

                try
                {
                    string versionWeb = client.DownloadString("http://sean-walton.co.uk/updates/bngmapoptions.php"); //Connect to the server and download the data.

                    if (versionWeb != null && versionWeb != "") //If we actually have content retrieved.
                    {
                        //Console.WriteLine(versionWeb);
                        string[] updateData = versionWeb.Split('|'); //The data online is split with | characters, so we explode that and are left with an array.

                        double versionWebNumber;

                        if (double.TryParse(updateData[0], out versionWebNumber)) //Try parse the double for the version number.
                        {
                            if (version < versionWebNumber) //If our version is less than on the web, we have an update.
                            {
                                MessageBoxResult result = MessageBox.Show("There's an update available for this program (current: " + version + ", available: " + updateData[0] + "). Would you like to go to the URL to download it now?\n\n(" + updateData[1] + ")", "Update available", MessageBoxButton.YesNo, MessageBoxImage.Information);

                                if (result == MessageBoxResult.Yes)
                                {
                                    Process.Start(updateData[1]); //Load the webpage for the second section of the downloaded string. In this case, it's the forum post.
                                }
                            }
                        }
                    }
                }
                catch (WebException ex)
                {
                    if (ex.Message.Contains("404")) //If we have a 404. Server is responding but the page isn't.
                    {
                        try
                        {
                            string attemptConnection = client.DownloadString("http://google.com"); //Of course, we check google. If google's down then in most cases the user doesn't have net.

                            if (attemptConnection != null && attemptConnection != "")
                            {
                                //I'm nice, so I don't blame the user.
                                MessageBox.Show("No updates can be gathered from the server at this time as it apears to be down, so you may have to check for updates manually.", "Update error", MessageBoxButton.OK, MessageBoxImage.Error);
                            }
                        }
                        catch { }
                    }
                }
                catch (Exception ex)
                {
                    //Any other exception and I have no idea, so just show a generic error.
                    MessageBox.Show("An unknown error occured attempting to check for an update. You may want to check for updates manually at this point. (" + ex.Message + ")", "Update error", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }));

            versionThread.Start(); //Of course, we need to start the thread. So we leave it to do its thing and report back later.
        }
    }
}
